@@ -31,6 +31,7 @@ import sys
 import threading
 import typing
 import os
+import re
 
 try:
     # Python 3.7
@@ -1263,36 +1264,35 @@ class UciProtocol(EngineProtocol):
                     self.info.update(_parse_uci_info(arg, engine.board, info))
 
             def _bestmove(self, engine: UciProtocol, arg: str) -> None:
-                try:
-                    if self.pondering:
-                        self.pondering = False
-                    elif not self.result.cancelled():
-                        tokens = arg.split(None, 2)
+                if self.pondering:
+                    self.pondering = False
+                elif not self.result.cancelled():
+                    tokens = arg.split(None, 2)
 
-                        bestmove = None
-                        if tokens[0] != "(none)":
-                            try:
-                                bestmove = engine.board.parse_uci(tokens[0])
-                            except ValueError as err:
-                                raise EngineError(err)
+                    bestmove = None
+                    if tokens[0] != "(none)":
+                        try:
+                            bestmove = engine.board.parse_uci(tokens[0])
+                        except ValueError as err:
+                            raise EngineError(err)
 
-                        pondermove = None
-                        if bestmove is not None and len(tokens) >= 3 and tokens[1] == "ponder" and tokens[2] != "(none)":
-                            engine.board.push(bestmove)
-                            try:
-                                pondermove = engine.board.push_uci(tokens[2])
-                            except ValueError:
-                                LOGGER.exception("engine sent invalid ponder move")
+                    pondermove = None
+                    if bestmove is not None and len(tokens) >= 3 and tokens[1] == "ponder" and tokens[2] != "(none)":
+                        engine.board.push(bestmove)
+                        try:
+                            pondermove = engine.board.push_uci(tokens[2])
+                        except ValueError:
+                            LOGGER.exception("engine sent invalid ponder move")
 
-                        self.result.set_result(PlayResult(bestmove, pondermove, self.info))
+                    self.result.set_result(PlayResult(bestmove, pondermove, self.info))
 
-                        if ponder and pondermove:
-                            self.pondering = True
-                            engine._position(engine.board)
-                            engine._go(limit, ponder=True)
-                finally:
-                    if not self.pondering:
-                        self.end(engine)
+                    if ponder and pondermove:
+                        self.pondering = True
+                        engine._position(engine.board)
+                        engine._go(limit, ponder=True)
+
+                if not self.pondering:
+                    self.end(engine)
 
             def end(self, engine: UciProtocol) -> None:
                 self.set_finished()
@@ -1546,6 +1546,9 @@ class UciOptionMap(MutableMapping[str, T]):
         return "{}({!r})".format(type(self).__name__, dict(self.items()))
 
 
+XBOARD_ERROR_REGEX = re.compile(r"^\s*(Error|Illegal move)(\s*\([^()]+\))?\s*:")
+
+
 class XBoardProtocol(EngineProtocol):
     """
     An implementation of the
@@ -1586,6 +1589,8 @@ class XBoardProtocol(EngineProtocol):
                     pass
                 elif line.startswith("feature "):
                     self._feature(engine, line.split(" ", 1)[1])
+                elif XBOARD_ERROR_REGEX.match(line):
+                    raise EngineError(line)
 
             def _feature(self, engine: XBoardProtocol, arg: str) -> None:
                 for feature in shlex.split(arg):
@@ -1730,6 +1735,8 @@ class XBoardProtocol(EngineProtocol):
                     self.set_finished()
                 elif not line.startswith("#"):
                     LOGGER.warning("%s: Unexpected engine output: %s", engine, line)
+                elif XBOARD_ERROR_REGEX.match(line):
+                    raise EngineError(line)
 
         return await self.communicate(Command)
 
@@ -1804,6 +1811,9 @@ class XBoardProtocol(EngineProtocol):
                     self._ping_after_move(engine)
                 elif line.startswith("#"):
                     pass
+                elif XBOARD_ERROR_REGEX.match(line):
+                    engine.first_game = True  # Board state might no longer be in sync
+                    raise EngineError(line)
                 elif len(line.split()) >= 4 and line.lstrip()[0].isdigit():
                     self._post(engine, line)
                 else:
@@ -1904,6 +1914,9 @@ class XBoardProtocol(EngineProtocol):
                     self._post(engine, line)
                 elif line == self.final_pong:
                     self.end(engine)
+                elif XBOARD_ERROR_REGEX.match(line):
+                    engine.first_game = True  # Board state might no longer be in sync
+                    raise EngineError(line)
                 else:
                     LOGGER.warning("%s: Unexpected engine output: %s", engine, line)
 
